import { RequestHandler, Router } from 'express';
import { Services } from '../../app';
import {
  CreateUser,
  ReadUser,
  LoginUser,
  UpdateUser,
} from '../../functions/user';
import {
  LoginUserSchema,
  CreateUserSchema,
  ReadUserSchema,
  LogoutUserSchema,
  UpdateUserSchema,
} from './schema/user';
import { CheckPermissions, getRequestId, ValidateSchema } from './middleware';

function LoginHandler(services: Services): RequestHandler {
  const login = LoginUser(services);

  return async (req, res, next) => {
    try {
      const user = await login(getRequestId(req), req.body);
      user ? (req.session.user = user) : (req.session.user = null);
      res.status(200).send(user);
    } catch (error) {
      next({ status: 401, message: 'wrong username or password' });
    }
  };
}

function LogoutHandler(services: Services): RequestHandler {
  return async (req, res, next) => {
    if (req.session.user) {
      req.session.user = null;
      res.status(204).send();
    } else {
      next({ status: 401, message: 'Not Logged In' });
    }
  };
}

function CreateHandler(services: Services): RequestHandler {
  const createUser = CreateUser(services);

  return async (req, res, next) => {
    try {
      const user = await createUser(getRequestId(req), req.body);
      res.status(201).send(user);
    } catch (error) {
      next({ status: 409, message: 'User Already Exists' });
    }
  };
}

function ReadHandler(services: Services): RequestHandler {
  const readUser = ReadUser(services);

  return async (req, res, next) => {
    try {
      const user = await readUser(getRequestId(req), req.params.uuid);
      res.status(200).send(user);
    } catch (error) {
      next({ status: 404, message: 'User Not Found' });
    }
  };
}

function UpdateHandler(services: Services): RequestHandler {
  const updateUser = UpdateUser(services);

  return async (req, res, next) => {
    try {
      const user = await updateUser(
        getRequestId(req),
        req.params.uuid,
        req.body,
      );
      res.status(200).send(user);
    } catch (error) {
      next({ status: 404, message: 'User Not Found' });
    }
  };
}

export function Routes(services: Services) {
  const router = Router();

  router.post(
    '/login',
    LoginUserSchema(),
    ValidateSchema(),
    LoginHandler(services),
  );
  router.post(
    '/logout',
    LogoutUserSchema(),
    ValidateSchema(),
    LogoutHandler(services),
  );
  router.get(
    '/read/:uuid',
    CheckPermissions(),
    ReadUserSchema(),
    ValidateSchema(),
    ReadHandler(services),
  );

  router.post(
    '/create',
    CheckPermissions(),
    CreateUserSchema(),
    ValidateSchema(),
    CreateHandler(services),
  );

  router.patch(
    '/update/:uuid',
    CheckPermissions(),
    UpdateUserSchema(),
    ValidateSchema(),
    UpdateHandler(services),
  );

  return router;
}
